/*
 * Copyright (c) 2008-2012 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
Sonatype.repoServer.CrowdConfigPanel = function(config) {
  var URL_REGEXP = /^(?:http|https):\/\//i;
  var config = config || {};
  var defaultConfig = {
    title : 'Crowd Configuration'
  };
  Ext.apply(this, config, defaultConfig);

  this.servicePath = {
    config : Sonatype.config.servicePath + '/crowd/config',
    connectionTest : Sonatype.config.servicePath + '/crowd/test'
  };

  this.referenceData = {
    config : {
      applicationName : '',
      applicationPassword : '',
      crowdServerUrl : '',
      sessionKeyIsAuthenticated : '',
      sessionKeyTokenKey : '',
      sessionValidationInterval : 0,
      sessionKeyLastValidation : '',
      useGroups : false,
      httpProxyHost : '',
      httpProxyPort : 0,
      httpProxyUsername : '',
      httpProxyPassword : '',
      httpMaxConnections : 0,
      httpTimeout : 0,
      cookieTokenKey : ''
    }
  };

  this.formPanel = new Ext.FormPanel({
        region : 'center',
        trackResetOnLoad : true,
        autoScroll : true,
        border : false,
        frame : true,
        collapsible : false,
        collapsed : false,
        labelWidth : 175,
        layoutConfig : {
          labelSeparator : ''
        },
        items : [{
              xtype : 'panel',
              buttonAlign : 'left',
              items : [{
                    xtype : 'fieldset',
                    checkboxToggle : false,
                    title : 'Access Settings',
                    anchor : Sonatype.view.FIELDSET_OFFSET_WITH_SCROLL,
                    collapsible : true,
                    autoHeight : true,
                    layoutConfig : {
                      labelSeparator : ''
                    },

                    items : [{
                          xtype : 'textfield',
                          fieldLabel : 'Application Name',
                          helpText : 'The name that the application will use when authenticating with the Crowd server.',
                          name : 'applicationName',
                          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
                          allowBlank : false,
                          itemCls : 'required-field'
                        }, {
                          xtype : 'textfield',
                          inputType : 'password',
                          fieldLabel : 'Application Password',
                          helpText : 'The password that the application will use when authenticating with the Crowd server.',
                          name : 'applicationPassword',
                          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
                          allowBlank : false,
                          itemCls : 'required-field'
                        }, {
                          xtype : 'textfield',
                          fieldLabel : 'Crowd Server URL',
                          helpText : 'The URL to use when connecting with the integration libraries to communicate with the Crowd server.',
                          name : 'crowdServerUrl',
                          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
                          allowBlank : false,
                          itemCls : 'required-field',
                          validator : function(v) {
                            if (v.match(URL_REGEXP))
                            {
                              return true;
                            }
                            else
                            {
                              return 'Protocol must be http:// or https://';
                            }
                          }
                        }, {
                          xtype : 'numberfield',
                          fieldLabel : 'Authentication Interval',
                          helpText : 'The number of minutes between authentication validation. If this value is set to 0, each HTTP request will be authenticated.',
                          name : 'sessionValidationInterval',
                          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
                          allowBlank : false,
                          itemCls : 'required-field'
                        }, {
                          xtype : 'checkbox',
                          fieldLabel : 'Use Groups',
                          helpText : 'Use Crowd Groups if checked, otherwise will use Crowd Roles.',
                          name : 'useGroups',
                          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL
                        }]
                  }, {
                    xtype : 'fieldset',
                    checkboxToggle : false,
                    title : 'HTTP Settings',
                    anchor : Sonatype.view.FIELDSET_OFFSET_WITH_SCROLL,
                    collapsible : true,
                    autoHeight : true,
                    layoutConfig : {
                      labelSeparator : ''
                    },

                    items : [{
                          xtype : 'numberfield',
                          fieldLabel : 'HTTP Max Connections',
                          helpText : 'The maximum number of HTTP connections in the connection pool for communication with the Crowd server.',
                          name : 'httpMaxConnections',
                          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
                          allowBlank : true
                        }, {
                          xtype : 'numberfield',
                          fieldLabel : 'HTTP Timeout',
                          helpText : 'The HTTP connection timeout (milliseconds) used for communication with the Crowd server. A value of zero indicates that there is no connection timeout.',
                          name : 'httpTimeout',
                          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
                          allowBlank : true
                        }]
                  }, {
                    xtype : 'fieldset',
                    checkboxToggle : false,
                    title : 'HTTP Proxy Settings',
                    anchor : Sonatype.view.FIELDSET_OFFSET_WITH_SCROLL,
                    collapsible : true,
                    autoHeight : true,
                    layoutConfig : {
                      labelSeparator : ''
                    },

                    items : [{
                          xtype : 'textfield',
                          fieldLabel : 'HTTP Proxy Host',
                          helpText : 'The name of the proxy server used to transport SOAP traffic to the Crowd server.',
                          name : 'httpProxyHost',
                          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
                          allowBlank : true
                        }, {
                          xtype : 'numberfield',
                          fieldLabel : 'HTTP Proxy Port',
                          helpText : 'The connection port of the proxy server (must be specified if a proxy host is specified).',
                          name : 'httpProxyPort',
                          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
                          allowBlank : true
                        }, {
                          xtype : 'textfield',
                          fieldLabel : 'HTTP Proxy Username',
                          helpText : 'The username used to authenticate with the proxy server (if the proxy server requires authentication).',
                          name : 'httpProxyUsername',
                          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
                          allowBlank : true
                        }, {
                          xtype : 'textfield',
                          inputType : 'password',
                          fieldLabel : 'HTTP Proxy Password',
                          helpText : 'The password used to authenticate with the proxy server (if the proxy server requires authentication).',
                          name : 'httpProxyPassword',
                          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
                          allowBlank : true
                        }]
                  }],
              buttons : [{
                    text : 'Test Connection',
                    minWidth : 120,
                    handler : this.testConnectionButtonHandler,
                    scope : this
                  }]
            }],
        buttons : [{
              text : 'Save',
              handler : this.saveButtonHandler,
              disabled : true,
              scope : this
            }, {
              text : 'Cancel',
              handler : this.cancelButtonHandler,
              scope : this
            }]
      });

  Sonatype.repoServer.CrowdConfigPanel.superclass.constructor.call(this, {
        autoScroll : false,
        layout : 'border',
        items : [this.formPanel]
      });

  this.formPanel.on('beforerender', this.beforeRenderHandler, this.formPanel);
  this.formPanel.on('afterlayout', this.afterLayoutHandler, this, {
        single : true
      });
  this.formPanel.form.on('actioncomplete', this.actionCompleteHandler, this);
  this.formPanel.form.on('actionfailed', this.actionFailedHandler, this);
};

Ext.extend(Sonatype.repoServer.CrowdConfigPanel, Ext.Panel, {
      beforeRenderHandler : function() {
        var sp = Sonatype.lib.Permissions;
        if (sp.checkPermission('nexus:crowdconfig', sp.EDIT))
        {
          this.buttons[0].disabled = false;
        }
      },

      actionCompleteHandler : function(form, action) {
        if (action.type == 'sonatypeSubmit')
        {
          if (action.options.url == this.servicePath.connectionTest)
          {
            Sonatype.MessageBox.alert('Test Connection', 'Crowd connection and authentication test completed successfully.').setIcon(Sonatype.MessageBox.INFO);
          }
        }
      },

      // (Ext.form.BasicForm, Ext.form.Action)
      actionFailedHandler : function(form, action) {
        if (action.options.url == this.servicePath.connectionTest)
        {
          var errorOptions = {
            hideErrorStatus : true
          };
          Sonatype.utils.connectionError(action.response, null, null, errorOptions);
        }
        else
        {
          if (action.failureType == null)
          {
            Sonatype.utils.connectionError(action.response, null, null, action.options);
          }
          else if (action.failureType == Ext.form.Action.CLIENT_INVALID)
          {
            Sonatype.MessageBox.alert('Missing or Invalid Fields', 'Please change the missing or invalid fields.').setIcon(Sonatype.MessageBox.WARNING);
          }
          else if (action.failureType == Ext.form.Action.CONNECT_FAILURE)
          {
            Sonatype.utils.connectionError(action.response, 'There is an error communicating with the server.')
          }
          else if (action.failureType == Ext.form.Action.LOAD_FAILURE)
          {
            Sonatype.MessageBox.alert('Load Failure', 'The data failed to load from the server.').setIcon(Sonatype.MessageBox.ERROR);
          }
        }
      },

      testConnectionButtonHandler : function() {
        var form = this.formPanel.form;

        if (!form.isValid())
          return;

        form.doAction('sonatypeSubmit', {
              method : 'PUT',
              url : this.servicePath.connectionTest,
              waitMsg : 'Testing Crowd connection and authentication...',
              fpanel : this.formPanel,
              serviceDataObj : this.referenceData.config,
              hideErrorStatus : true,
              autoValidation : false
            });
      },

      saveButtonHandler : function() {
        var form = this.formPanel.form;

        if (!form.isValid())
          return;

        form.doAction('sonatypeSubmit', {
              method : 'PUT',
              url : this.servicePath.config,
              waitMsg : 'Updating Crowd configuration...',
              fpanel : this.formPanel,
              serviceDataObj : this.referenceData.config
            });
      },

      cancelButtonHandler : function() {
        Sonatype.view.mainTabPanel.remove(this.id, true);
      },

      afterLayoutHandler : function() {

        // invoke form data load
        this.formPanel.getForm().doAction('sonatypeLoad', {
              url : this.servicePath.config,
              method : 'GET',
              fpanel : this.formPanel
            });

        // register required field quicktip, but have to wait for elements to
        // show up in DOM
        var temp = function() {
          var els = Ext.select('.required-field .x-form-item-label', this.formPanel.getEl());
          els.each(function(el, els, i) {
                Ext.QuickTips.register({
                      target : el,
                      cls : 'required-field',
                      title : '',
                      text : 'Required Field',
                      enabled : true
                    });
              });
        }.defer(300, this);
      }
    });

Sonatype.Events.addListener('nexusNavigationInit', function(nexusPanel) {
      nexusPanel.add({
            enabled : Sonatype.lib.Permissions.checkPermission('nexus:crowdconfig', Sonatype.lib.Permissions.READ),
            sectionId : 'st-nexus-security',
            title : 'Crowd Configuration',
            tabId : 'crowd-configuration',
            tabCode : Sonatype.repoServer.CrowdConfigPanel
          });
    });
