/*
 * Copyright (c) 2008-2012 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */

$(document).ready(function() {
  "use strict";

  var
        // objects
        licenseBaseUrl = $('#license-url').attr('href'),
        urlPath = '/rest/license/',
        licenseUrl = licenseBaseUrl + urlPath,
        // functions
        invalid, clearInvalid, showError, mask, unmask, showAgreementWindow, uploadB64;

  if ($.browser.mozilla && $.browser.version >= 13) {
    // move file input out of the fake link span (will repeat click events otherwise) and relay click events
    $('body')[0].appendChild($('#file')[0]);
    $('#file').css('right', '100px');
    $('.fake-link').click(function() {
      setTimeout(function() {
        $('input[name=file]').click();
      },100);
      return false;
    });
  }

  /* UI setup */
  invalid = function(msg) {
    $('#error')[0].innerHTML = msg;
  };
  clearInvalid = function() {
    invalid('');
  };
  showError = function(data) {
    if (data.errors) {
      if (data.errors[0].id === 'file') {
        invalid(data.errors[0].msg);
      } else {
        invalid(data.errors);
      }
    } else {
      invalid('Could not activate the license. Check the Nexus logs for further information or <a target="_blank" href="http://links.sonatype.com/products/nexus/pro-eval/support-request">contact us</a>.');
    }
    // signal whether we actually had an error
    return data.errors;
  };

  mask = function(msg) {
    $('body').mask(msg);
  };
  unmask = function() {
    $('body').unmask();
  };

  showAgreementWindow = function(agreeFn, arg) {
    var agreementWindow = $('#license-agreement');
    agreementWindow.find('.content').load('../LICENSE.html');
    agreementWindow.dialog({
      title : 'Nexus Professional License Agreement',
      width : 600,
      height : 500,
      modal : true,
      show : function() {
        // remove css classes from buttons to have the default style
        var buttons = $(this).find('.ui-button');
        buttons.removeClass().unbind('mouseenter mouseleave mousedown focus');
        buttons[0].className = 'download-license-button';
        $(this).focus();
      },
      buttons : {
        'Download a copy of the license.' : function(evt) {
          window.open('../LICENSE.html', '_new');
        },
        'I Agree' : function(evt) {
          agreementWindow.dialog('close');
          agreementWindow.dialog('destroy');
          agreeFn(arg);
        },
        'I Don\'t Agree' : function(evt) {
          agreementWindow.dialog('close');
          agreementWindow.dialog('destroy');
        }
      }
    });

    // content is at the bottom after first dialog view, so scroll to top explicitely
    $('#license-agreement').scrollTop(0);
  };

  /* Upload Controllers */
  uploadB64 = function(fp) {
    mask('Activating license...');
    $.ajax('../service/local/licensing/upload', {
      type : 'POST',
      cache : false,
      contentType : 'text/plain',
      data : fp,
      headers : {
        'Accept' : 'application/json'
      },
      success : function(data, textStatus, xhr) {
        window.location = '../index.html';
      },
      error : function(xhr, status, error) {
        if (!xhr.responseText || !showError(JSON.parse(xhr.responseText))) {
          if (error) {
            invalid('Could not activate the license: ' + error);
          } else {
            invalid('Could not activate the license. Check the Nexus logs for further information or <a target="_blank" href="http://links.sonatype.com/products/nexus/pro-eval/support-request">contact us</a>.');
          }
        }
      },
      complete : function() {
        unmask();
      }
    });
  };

  $(function() {
    var msie = $.browser.msie !== undefined;
    $('input[name=file]').fileupload({
      dataType : 'json',
      // change URL for IE upload to switch behavior in the REST resource
      url : "../service/local/licensing/upload" + (msie?"/ie":""),
      add : function(e, data) {
        showAgreementWindow(function() {
          mask("Uploading License...");
          data.submit();
        });
      },
      fail : function(e, data) {
        unmask();
        var json = $.parseJSON(data.jqXHR.responseText);
        showError(json||{});
      },
      done : function(e, data) {
        unmask();

        var json = data.result;

        // we may get here for iframe transport + certain error codes as well
        // IE will treat at least 400 and 500 status as CORS (because it loads a local error page)
        // and will get here without any response object
        if ( !json || !json.success ) {
          showError(json||{});
          return;
        }

        window.location = '../index.html';
      }
    });
  });

  /**
   * License activation. We either have fingerprint or BASE64 content.
   */
  $('#activate').click(function(evt) {
    var
          // objects
          urlPath = '/rest/license/',
          licenseUrl = licenseBaseUrl + urlPath,
          fp = $.trim($('#license-id').val()),
          // functions
          getLicenseData;

    getLicenseData = function(callback) {
      var
            // objects
            connectors,
            // functions
            xhrConnector, proxyConnector, _tryGet;
      /*
       Two ways to get license data: jQuery ajax (for FF, webkit, plus XDomain stuff contributed by jQ plugin for newer IE), or use Nexus instance as "proxy".
       */
      connectors = [];

      xhrConnector = function(fp) {
        $.ajax(licenseUrl + fp, {
          type : 'GET',
          cache : false,
          headers : {
            'Accept' : 'application/json'
          },
          complete : function(xhr, textStatus) {
            if (textStatus === 'success' && xhr) {
              _tryGet(xhr.responseText);
            } else {
              _tryGet();
            }
          }
        });
      };
      connectors.push(xhrConnector);

      proxyConnector = function() {
        $.ajax("../service/local/licensing/upload/" + fp, {
          type : 'GET',
          cache : false,
          headers : {
            'Accept' : 'application/json'
          },
          complete : function(xhr, textStatus) {
            if ( textStatus === 'success' ) {
              _tryGet(xhr.responseText);
            } else {
              _tryGet();
            }
          }
        });
      };
      connectors.push(proxyConnector);

      _tryGet = function(data) {
        if (data !== undefined) {
          callback(data);
        } else if (connectors.length !== 0) {
          // take and remove first connector and execute it
          (connectors.splice(0, 1)[0])(fp);
        } else {
          callback();
        }
      };

      _tryGet();
    };

    clearInvalid();
    if (fp.length === 0) {
      invalid('Please enter your license key.');
    } else if (fp.length === 40) {
      mask('Verifying license id...');
      getLicenseData(function(data) {
              var json;

              unmask();

              if (data !== undefined) {
                json = $.parseJSON(data);
                if (json.license) {
                  showAgreementWindow(uploadB64, json.license);
                } else {
                  // we got a message, but no license data contained
                  if ( console ) {
                    console.log(data);
                  }
                  invalid('License could not be activated. Please <a target="_blank" href="http://links.sonatype.com/products/nexus/pro-eval/support-request">contact us</a>.');
                }
              } else {
                // with CORS, browser do not expose any information about the response status -> generic 'contact us'.
                invalid('License could not be activated. Please <a target="_blank" href="http://links.sonatype.com/products/nexus/pro-eval/support-request">contact us</a>.');
              }
            });
    } else if (fp.length > 40) {
      showAgreementWindow(uploadB64, fp);
    } else {
      invalid('This does not appear to be a valid license key. If you continue to experience problems activating your license, please <a target="_blank" href="http://links.sonatype.com/products/nexus/pro-eval/support-request">contact us</a>.');
    }
  });

});
